import path from 'path-browserify'
import useI18n from '@/i18n'
import type { RouteRecordRaw } from 'vue-router'

interface IData {
  path: string
  title: Array<string>
}

/**
 * 筛选出可供搜索的路由对象
 * @param routes 路由表
 * @param basePath 基础路径，默认为 /
 * @param prefixTitle
 */
export const generateRoutes = (
  routes: RouteRecordRaw[],
  basePath: string = '/',
  prefixTitle: string[] = []
) => {
  // 获取 i18n 实例
  const i18n = useI18n()

  // 创建 result 数据
  let res: any[] = []
  // 循环 routes 路由
  for (const route of routes) {
    // 创建包含 path 和 title 的 item
    const data: IData = {
      // path.resolve 拼接字符串时遇到相邻的重叠部分会合并
      // path.join 只是简单的字符串拼接
      path: path.resolve(basePath, route.path),
      title: [...prefixTitle]
    }
    // 当前存在 meta 时，使用 i18n 解析国际化数据，组合成新的 title 内容
    const reg = /.*\/:.*/ // 匹配动态路由的正则
    // 过滤动态路由: 动态路由不允许被搜索
    if (
      route?.meta?.title &&
      !reg.test(route.path) &&
      !res.find((item) => item.path === data.path)
    ) {
      // 标题国际化处理
      const i18nTitle = i18n.global.t(`route.${route.meta.title}`)
      data.title = [...data.title, i18nTitle]
      res.push(data)
    }

    // 存在 children 时，迭代调用
    if (route.children) {
      const tempRoutes = generateRoutes(route.children, data.path, data.title)
      if (tempRoutes.length > 0) {
        res = [...res, ...tempRoutes]
      }
    }
  }
  return res
}
