// import axios, { AxiosHeaders } from 'axios'
import axios from 'axios'
import type { AxiosInstance, AxiosRequestConfig } from 'axios'
import type { HYRequestConfig } from './type'
// import { localCache } from '@/utils/cache'
// import { LOGIN_TOKEN } from '@/global/constants'

// 拦截器: 蒙版Loading/token/修改配置

/**
 * 两个难点:
 *  1.拦截器进行精细控制
 *    > 全局拦截器
 *    > 实例拦截器
 *    > 单次请求拦截器
 *
 *  2.响应结果的类型处理(泛型)
 */

class HYRequest {
  instance: AxiosInstance

  // request实例 => axios的实例
  constructor(config: HYRequestConfig) {
    this.instance = axios.create(config)

    // 每个instance实例都添加拦截器
    this.instance.interceptors.request.use(
      (config) => {
        // loading/token
        return config
      },
      (err) => {
        return err
      }
    )

    this.instance.interceptors.response.use(
      (res) => {
        return res.data
      },
      (err) => {
        return err
      }
    )

    // 针对特定的hyRequest实例添加拦截器
    this.instance.interceptors.request.use(
      config.interceptors?.requestSuccessFn,
      config.interceptors?.requestFailureFn
    )
    this.instance.interceptors.response.use(
      config.interceptors?.responseSuccessFn,
      config.interceptors?.responseFailureFn
    )
  }

  // 封装网络请求的方法
  // T => IHomeData
  request<T = any>(config: HYRequestConfig<T>) {
    // 单次请求的成功拦截处理
    if (config.interceptors?.requestSuccessFn) {
      // 类型断言 as：指明更加具体的类型或者不太具体的类型
      config = config.interceptors.requestSuccessFn(
        config as any
      ) as HYRequestConfig<T>
    }

    // 返回Promise
    return new Promise<T>((resolve, reject) => {
      this.instance
        .request<any, T>(config)
        .then((res) => {
          // 单词响应的成功拦截处理
          if (config.interceptors?.responseSuccessFn) {
            res = config.interceptors.responseSuccessFn(res)
          }
          resolve(res)
        })
        .catch((err) => {
          reject(err)
        })
    })
  }

  get<T = any>(config: AxiosRequestConfig<T>) {
    return this.request({
      // headers: new AxiosHeaders({
      //   Authorization: localCache.getCache(LOGIN_TOKEN)
      // }),
      ...config,
      method: 'GET'
    })
  }
  post<T = any>(config: AxiosRequestConfig<T>) {
    return this.request({
      ...config,
      method: 'POST'
    })
  }
  delete<T = any>(config: AxiosRequestConfig<T>) {
    return this.request({
      ...config,
      method: 'DELETE'
    })
  }
  patch<T = any>(config: AxiosRequestConfig<T>) {
    return this.request({
      ...config,
      method: 'PATCH'
    })
  }
}

export default HYRequest
