import {
  APP_STORE_CACHE_KEY,
  DEFAULT_LOCALE_LANGUAGE
} from '@/global/constants'
import { defineStore } from 'pinia'

interface IState {
  language: string
  tagsViewList: any[]
}

type RemoveTagType = 'other' | 'right' | 'index'

interface IRemoveTagPayload {
  type: RemoveTagType
  index: number
}

const useAppStore = defineStore('app', {
  state: (): IState => ({
    // 当前使用的语言
    language: DEFAULT_LOCALE_LANGUAGE,
    // tagView 数据
    tagsViewList: []
  }),
  getters: {},
  actions: {
    /**
     * 设置国际化语言
     * @param lang 当前使用语言
     */
    setLanguage(lang: string) {
      this.language = lang
    },
    /**
     * 添加 tag
     * @param tag 路由数据对象
     */
    addTagsViewList(tag: any) {
      const isFind = this.tagsViewList.find(
        (item: any) => item.path === tag.path
      )
      // 处理重复
      if (!isFind) {
        this.tagsViewList.push(tag)
      }
    },
    /**
     * 为指定的 tag 修改 title
     */
    changeTagsView(index: number, tag: any) {
      this.tagsViewList[index] = tag
    },
    /**
     * 移除 tag
     * @param type 删除操作的类型
     */
    removeTagsView({ type, index }: IRemoveTagPayload) {
      switch (type) {
        case 'index': {
          this.tagsViewList.splice(index, 1) // 删除 index 位置的 tag
          break
        }
        case 'right': {
          this.tagsViewList.splice(index + 1) // 移除 index 右侧所有 tag
          break
        }
        case 'other': {
          this.tagsViewList.splice(index + 1) // 删除右侧 tag
          this.tagsViewList.splice(0, index) // 再删除左侧 tag
          break
        }
      }
    }
  },
  // 启用数据持久化
  persist: {
    key: APP_STORE_CACHE_KEY
  }
})

export default useAppStore
