import { defineStore } from 'pinia'
import { publicRoutes, privateRoutes } from '@/router'
import type { RouteRecordRaw } from 'vue-router'
import { PERMISSION_STORE_CACHE_KEY } from '@/global/constants'
import type { IState } from './types'

const usePermissionStore = defineStore('permission', {
  state: (): IState => ({
    // 路由表：初始时拥有的路由表
    routes: publicRoutes
  }),
  actions: {
    /**
     * 增加路由
     */
    setRoutes(newRoutes: RouteRecordRaw[]) {
      this.routes = [...publicRoutes, ...newRoutes]
    },
    /**
     * 根据权限数据筛选路由
     * @param menus 权限数据
     */
    filterRoutes(menus: string[]) {
      const routes: any[] = []

      // 根据 menu 与路由对象的 name 字段一一匹配的关系，添加权限路由
      menus.forEach((key) => {
        routes.push(
          ...privateRoutes.filter((item: RouteRecordRaw) => item.name === key)
        )
      })

      // 所有不匹配的路由，全部进入 404 的路由配置
      // ? 注意：404 页面的配置必须要在路由表的最后一个
      // ? 现在似乎并不需要是最后一个，只要配置了就行
      // routes.push({
      //   path: '/:catchAll(.*)',
      //   redirect: '/404'
      // })

      this.setRoutes(routes)

      return routes
    }
  },
  persist: {
    key: PERMISSION_STORE_CACHE_KEY
  }
})

export default usePermissionStore
