import { watch } from 'vue'
import useI18n from '@/i18n'
import useAppStore from '@/stores/app'

/**
 * 获取国际化后的 title
 * @param title 标题字段名称
 * @returns 国际化后的 title
 */
export const generateTitle = (title: string) => {
  const i18n = useI18n()
  return i18n.global.t(`route.${title}`)
}

/**
 * 监听语言切换, 同时执行 cb
 */
export const watchSwitchLang = (...cbs: any[]) => {
  const appStore = useAppStore()
  watch(
    () => appStore.language,
    () => {
      // 当语言切换时, 执行所有传入的回调函数
      cbs.forEach((cb) => cb(appStore.language))
    }
  )
}
