/**
 * 1. 动态的列数据
 * 2. 被勾选的动态列数据
 * 3. table 的列数据
 */
import { ref, watch } from 'vue'
import getDynamicData from './dynamic-data'
import { watchSwitchLang } from '@/utils/i18n'

// 暴露出动态的列数据
export const dynamicData = ref(getDynamicData())

// 被勾选的动态列数据标签
export const selectDynamicLabels = ref<string[]>([])
// 默认全部勾选
const initSelectDynamicLabel = () => {
  selectDynamicLabels.value = dynamicData.value.map((item) => item.label)
}
initSelectDynamicLabel()

// 声明 table 的列数据
export const tableColumns = ref<any[]>([])
watch(
  selectDynamicLabels,
  (val) => {
    tableColumns.value = []
    // 遍历列数据
    const selectedData = dynamicData.value.filter((item) => {
      return val.includes(item.label)
    })
    tableColumns.value.push(...selectedData)
  },
  {
    // 初始化时执行一次
    immediate: true
  }
)

// ! 语言切换时，重新初始化数据
watchSwitchLang(() => {
  // 重新获取国际化的值
  dynamicData.value = getDynamicData()
  // 重新处理被勾选的列数据
  initSelectDynamicLabel()
})
