import { ref, type Ref } from 'vue'
import Sortable from 'sortablejs'
import { updateArticleRanking } from '@/service/article'
import useI18n from '@/i18n'

export const tableRef = ref()

export const initSortable = (tableDataRef: Ref<any[]>, cb: () => void) => {
  // 1. 要拖拽的元素的父元素
  const el = tableRef.value?.$el.querySelector(
    '.el-table__body-wrapper table.el-table__body > tbody'
  )

  // 2. 配置对象
  Sortable.create(el, {
    // 拖拽元素时为元素添加的类名
    ghostClass: 'sortable-ghost',
    // 拖拽结束的回调方法
    async onEnd(event: any) {
      // 这里的 index 是从 0 开始的，传递给服务器的排序是从 1 开始的
      // 注意这里获取到的只是用户拖拽后的元素索引位置，并不是数据的真实排名
      // 需要先通过排序后的索引值获取到对应表格行的数据
      const { newIndex, oldIndex } = event
      await updateArticleRanking({
        initRanking: tableDataRef.value[oldIndex].ranking,
        finalRanking: tableDataRef.value[newIndex].ranking
      })

      ElMessage.success(useI18n().global.t('article.sortSuccess'))

      // element-plus 的表格在重新获取数据时，默认不会重新渲染
      // 解决方法：先将数据清空，然后重新请求数据，即可实现表格的重新渲染
      tableDataRef.value = []

      // 重新请求数据
      cb?.()
    }
  })
}
